#version 120

varying vec3 f_normal;
varying vec3 f_pos;

uniform int spotlights_count;
uniform sampler2D tex;

vec3 calc_light(vec3 light_dir, gl_LightProducts products)
{
	vec3 normal = normalize(f_normal);
	vec3 view_dir = normalize(vec3(0.0f, 0.0f, 0.0f) - f_pos);
	vec3 halfway_dir = normalize(light_dir + view_dir);
	
	vec3 ambient = vec3(products.ambient);
	vec3 diffuse = vec3(products.diffuse) * max(dot(normal, light_dir), 0.0);
	//specular nie jest odpowiednio ustawiany, wic byszczy si wszystko eby byo wida e potrafi :D
	//vec3 specular = vec3(products.specular) * pow(max(dot(normal, halfway_dir), 0.0), max(1.0, gl_FrontMaterial.shininess));
	vec3 specular = vec3(0.4f) * pow(max(dot(normal, halfway_dir), 0.0), 16.0);
	
	return ambient + diffuse + specular;
}


vec3 calc_point_light(gl_LightSourceParameters light, gl_LightProducts products)
{
	vec3 light_dir = normalize(vec3(light.position) - f_pos);
	vec3 val = calc_light(light_dir, products);
	
	float distance = length(vec3(light.position) - f_pos);
	float atten = 1.0f / (1.0f + light.linearAttenuation * distance + light.quadraticAttenuation * (distance * distance));
	
	return val * atten;
}

vec3 calc_spot_light(gl_LightSourceParameters light, gl_LightProducts products)
{
	vec3 light_dir = normalize(vec3(light.position) - f_pos);
	
	float theta = dot(light_dir, normalize(-light.spotDirection));
	float outerCutoff = light.spotCosCutoff - 0.03;
	float epsilon = light.spotCosCutoff - outerCutoff;
	float intensity = clamp((theta - outerCutoff) / epsilon, 0.0, 1.0);

	vec3 point = calc_point_light(light, products);
	return point * intensity;
}

vec3 calc_dir_light(gl_LightSourceParameters light, gl_LightProducts products)
{
	vec3 light_dir = normalize(vec3(-light.spotDirection));
	return calc_light(light_dir, products);
}

void main()
{
	vec3 result = vec3(0.0);
	result += calc_dir_light(gl_LightSource[0], gl_FrontLightProduct[0]);
	for (int i = 0; i < spotlights_count; i++)
		result += calc_spot_light(gl_LightSource[i + 1], gl_FrontLightProduct[i + 1]);
	result += vec3(gl_FrontMaterial.emission);
	
	vec4 tex_color;
	tex_color = texture2D(tex, gl_TexCoord[0].xy);
	
	gl_FragColor = vec4(result * tex_color.xyz, tex_color.w);
}
